@@ -4,33 +4,63 @@ import os
 import sys
 from importlib.util import module_from_spec, spec_from_file_location
 from itertools import chain
+from tabulate import tabulate
+
+from mc_nag.base_utils.exceptions import InvalidParameter
+from mc_nag.base_utils.models.rule_directory import RuleDirectoryManager
 from .printers import BCOLORS
 
+TABULATE_FORMAT = 'fancy_grid'
+
+
+TABULATE_FORMAT = 'fancy_grid'
+
 
 class RuleEvaluator:
     """Engine to evaluate rules.
 
     This class will find all rules inside the supplied standard rules
     directory and evaluate them against the given template model.
+
     :param model: Template data model built from a template.
+    :param rule_dir_manager: Required to pass an instance of the RuleDirectoryManager.
+    :param requested_tags: Optional tuple of tags to filter Rules.
     """
 
-    def __init__(self, model, standard_rules_dir, enable_standard_rules=True,
-                 custom_rules_dirs=None):
+    def __init__(self, model: str, rule_dir_manager: RuleDirectoryManager,
+                 rule_params=None, requested_tags: tuple = None):
         """Assign arguments as attributes and prepare rules."""
         self.model = model
-        self._standard_rules_dir = standard_rules_dir
-        self.rule_set = self._retrieve_rule_objects(enable_standard_rules,
-                                                    custom_rules_dirs)
+        try:
+            self.rule_dir_manager = rule_dir_manager
+        except AttributeError:
+            raise InvalidParameter(
+                "RuleEvaluator must be pass an instance of RuleDirectoryManager.")
+        self.requested_tags = set()
+        self._set_requested_tags(requested_tags)
+        self.rule_set = self._retrieve_rule_objects()
+        self.rule_params = rule_params
+
+    def _set_requested_tags(self, requested_tags):
+        if isinstance(requested_tags, str):
+            self.requested_tags.update([requested_tags])
+        if isinstance(requested_tags, (list, tuple)):
+            self.requested_tags.update(requested_tags)
 
     def evaluate_rules(self):
         """Evaluate given rules against the data model."""
         violations_by_rule = {}
         severity_counts = {}
 
-        for rule in self.rule_set:
+        # Allows tag-based Rule scanning.
+        if self.requested_tags:
+            rule_classes = self._filter_tags()
+        else:
+            rule_classes = self.rule_set
+
+        for rule in rule_classes:
             # Instantiate rule
-            rule_object = rule(self.model)
+            rule_object = rule(self.model, self.rule_params)
 
             # Evaluate rule
             violating_resources = rule_object.evaluate()
@@ -55,62 +85,57 @@ class RuleEvaluator:
         """Output list of rules and rule IDs in given format.
 
         :param output: Format in which the output should be.
-        :param verbose: Level of verbosity.
         """
+        if not self.rule_set:
+            return None
         # Dict to hold values for duplicate checking
-        dupe_rules = {}
+        rule_table = []
+        dupe_rules = []
         rev_dict = {}
 
-        # Gather table formatting information & print header
-        left_pad = max(max([len(rule.rule_id) for rule in self.rule_set]), 7)
-        right_pad = max(max([len(rule.__name__) for rule in self.rule_set]), 9)
-        print(f'{"Rule ID":{left_pad}} | Rule Name')
-        print(f"{''.rjust(left_pad, '-')} | {''.rjust(right_pad, '-')}")
+        # Gather table information & print
+        print(
+            tabulate(
+                [[rule.rule_id, rule.__name__]
+                    for rule in sorted(self.rule_set, key=lambda x: x.rule_id)],
+                headers=["Rule ID", "Rule Name"],
+                tablefmt=TABULATE_FORMAT
+            )
+        )
 
-        # Iterate over all found rules and build rev dict
+        # Find duplicate rule IDs
         for rule in sorted(self.rule_set, key=lambda x: x.rule_id):
-            print(f'{rule.rule_id:{left_pad}} | {rule.__name__}')
             rev_dict.setdefault(rule.rule_id, set()).add(
                 f'{rule.__name__} ({rule.__module__}.py)'
             )
-
-        # Find duplicate rule IDs
         for key, values in rev_dict.items():
             if len(values) > 1:
-                dupe_rules[key] = values
+                for value in values:
+                    dupe_rules.append([key, value])
 
         # Output
         if dupe_rules:
-            left_pad = max(max([len(key) for key in dupe_rules]), 17)
             print(f'\n{BCOLORS["ERROR"]}Found duplicate rule IDs!' +  # noqa: W504
                   f'{BCOLORS["ENDC"]}\n')
-            print(f'{"Duplicate Rule ID":{left_pad}} | Duplicate Rule Names')
-            for key, values in dupe_rules.items():
-                print(f"{''.rjust(left_pad, '-')} | {''.rjust(20, '-')}")
-                print(f"{key:{left_pad}} | " +  # noqa: W504
-                      f',\n{"".rjust(left_pad)} | '.join(values))
+
+            print(tabulate(dupe_rules, headers=["Duplicate Rule ID", "Duplicate Rule Names"],
+                           tablefmt="fancy_grid")
+                  )
         else:
             print(f'\n{BCOLORS["OKGREEN"]}No duplicates{BCOLORS["ENDC"]} 🎉')
 
         return dupe_rules
 
-    def _retrieve_rule_objects(self, enable_standard_rules=True,
-                               custom_rules_dirs=None):
+    def _retrieve_rule_objects(self):
         """Return list of rule classes to evaluate.
 
         Based on this StackOverflow info:
         https://stackoverflow.com/a/41904558
         """
-        paths = []
-
-        # Platform-specific standard rules path
-        if enable_standard_rules:
-            paths.append(self._standard_rules_dir)
-
-        # Add custom rules directories to paths if existent
-        if custom_rules_dirs:
-            paths += custom_rules_dirs
-
+        try:
+            paths = self.rule_dir_manager.rule_directories()
+        except AttributeError:
+            raise InvalidParameter("RuleEvaluator requires the `rule_dir_manager` parameter.")
         # Set to hold found rules
         rules = set()
         found_rule_modules = set()
@@ -144,3 +169,46 @@ class RuleEvaluator:
                     found_rule_modules.add(py_file)
 
         return rules
+
+    def display_tags(self):
+        """Gather tags from the rule set & print."""
+        categories = set()
+        sources = set()
+        for rule in self.rule_set:
+            # Category Tags:
+            try:
+                categories.update(rule.category_tags)
+            except AttributeError:
+                pass
+            # Resource Tags:
+            try:
+                sources.update(rule.source_tags)
+            except AttributeError:
+                pass
+
+        if categories or sources:
+            print(f'\n{BCOLORS["ERROR"]}List of Tags{BCOLORS["ENDC"]}\n')
+            if categories:
+                category_list = [[x] for x in categories]
+                print(tabulate(category_list, headers=["Category Tags"],
+                               tablefmt=TABULATE_FORMAT)
+                      )
+            if sources:
+                resource_list = [[x] for x in sources]
+                print(tabulate(resource_list, headers=["Source Tags"],
+                               tablefmt=TABULATE_FORMAT)
+                      )
+        else:
+            print(f'\n{BCOLORS["OKGREEN"]}No tags for your rule set.{BCOLORS["ENDC"]}')
+
+    def _filter_tags(self):
+        """Find only Rules with matching tags.
+
+        :return result:Returns a set of tag filtered Rule classes.
+        """
+        result = set()
+        for rule in self.rule_set:
+            all_tags_of_rule = rule.source_tags.union(rule.category_tags)
+            if all_tags_of_rule.intersection(self.requested_tags):
+                result.add(rule)
+        return result
