@@ -32,7 +32,7 @@ def test_base_rule_good_subclass():
 
 def test_base_rule_bad_subclass():
     """Sad Path: Catch error when subclass is missing attributes."""
-    # pylint: disable=too-few-public-methods
+    # pylint: disable=too-few-public-methods,abstract-method
     class RuleMissingAttributes(BaseRule):
         """Create subclass of BaseRule without setting all attributes."""
 
@@ -82,7 +82,7 @@ def test_tags_issue_warning():
 
 
 def test_tag_cleaning():
-    """Happy Path: """
+    """Happy Path: Make sure tags are lowercased and text only."""
     class TagCleaned(BaseRule):
         """Ensure that tags are cleaned and valid."""
 
