@@ -15,6 +15,8 @@ class CustomPlatformRule(BaseRule):
     url = 'https://github.com/stelligent/mc-nag/tests/rules/custom/' + \
           'custom_rule_for_option_testing.py'
     resolution = 'N/A'
+    category_tags = {"custom-resource"}
+    source_tags = {"stelligent"}
 
     def evaluate(self):
         """Find storageAccount resources and validate 'encryption' property.
