@@ -15,6 +15,8 @@ class StandardRule(BaseRule):
     url = 'https://github.com/stelligent/mc-nag/tests/rules/standard/' + \
           'standard_rule.py'
     resolution = 'N/A'
+    category_tags = {"custom-resource"}
+    source_tags = {"stelligent"}
 
     def evaluate(self):
         """Return empty list."""
